@@ -1,17 +1,19 @@
 # SPDX-FileCopyrightText: 2021 Division of Intelligent Medical Systems, DKFZ
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
-import numpy as np
-from typing import Union, Dict
 from abc import abstractmethod
-import gc
+from typing import Dict, Union
+
+import numpy as np
 
-from simpa.utils import Tags, Settings
 from simpa.core import SimulationModule
+from simpa.core.device_digital_twins import (IlluminationGeometryBase,
+                                             PhotoacousticDevice)
+from simpa.io_handling.io_hdf5 import load_data_field, save_hdf5
+from simpa.utils import Settings, Tags
 from simpa.utils.dict_path_manager import generate_dict_path
-from simpa.io_handling.io_hdf5 import save_hdf5, load_hdf5
-from simpa.core.device_digital_twins import IlluminationGeometryBase, PhotoacousticDevice
-from simpa.utils.quality_assurance.data_sanity_testing import assert_array_well_defined
+from simpa.utils.quality_assurance.data_sanity_testing import \
+    assert_array_well_defined
 
 
 class OpticalForwardModuleBase(SimulationModule):
@@ -57,16 +59,13 @@ class OpticalForwardModuleBase(SimulationModule):
 
         self.logger.info("Simulating the optical forward process...")
 
-        properties_path = generate_dict_path(Tags.SIMULATION_PROPERTIES,
-                                             wavelength=self.global_settings[Tags.WAVELENGTH])
+        file_path = self.global_settings[Tags.SIMPA_OUTPUT_PATH]
+        wl = str(self.global_settings[Tags.WAVELENGTH])
 
-        optical_properties = load_hdf5(self.global_settings[Tags.SIMPA_OUTPUT_PATH], properties_path)
-        absorption = optical_properties[Tags.DATA_FIELD_ABSORPTION_PER_CM][str(self.global_settings[Tags.WAVELENGTH])]
-        scattering = optical_properties[Tags.DATA_FIELD_SCATTERING_PER_CM][str(self.global_settings[Tags.WAVELENGTH])]
-        anisotropy = optical_properties[Tags.DATA_FIELD_ANISOTROPY][str(self.global_settings[Tags.WAVELENGTH])]
-        gruneisen_parameter = optical_properties[Tags.DATA_FIELD_GRUNEISEN_PARAMETER]
-        del optical_properties
-        gc.collect()
+        absorption = load_data_field(file_path, Tags.DATA_FIELD_ABSORPTION_PER_CM, wl)
+        scattering = load_data_field(file_path, Tags.DATA_FIELD_SCATTERING_PER_CM, wl)
+        anisotropy = load_data_field(file_path, Tags.DATA_FIELD_ANISOTROPY, wl)
+        gruneisen_parameter = load_data_field(file_path, Tags.DATA_FIELD_GRUNEISEN_PARAMETER)
 
         _device = None
         if isinstance(device, IlluminationGeometryBase):
