@@ -7,6 +7,7 @@ from simpa.utils.libraries.structure_library import Structures
 from simpa.utils import Tags
 import numpy as np
 from simpa.utils import create_deformation_settings
+import torch
 
 
 class ModelBasedVolumeCreationAdapter(VolumeCreatorModuleBase):
@@ -99,4 +100,7 @@ class ModelBasedVolumeCreationAdapter(VolumeCreatorModuleBase):
 
             global_volume_fractions[mask] += added_volume_fraction[mask]
 
+        # explicitly empty cache to free reserved GPU memory after volume creation
+        torch.cuda.empty_cache()
+
         return volumes
