@@ -25,3 +25,5 @@ class SegmentationClasses:
     HEAVY_WATER = 9
     COUPLING_ARTIFACT = 10
     MEDIPRENE = 11
+    SOFT_TISSUE = 12
+    LYMPH_NODE = 13
