@@ -1,6 +1,8 @@
 # SPDX-FileCopyrightText: 2021 Division of Intelligent Medical Systems, DKFZ
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
+
+import torch
 import numpy as np
 
 from simpa.utils import Tags
@@ -31,9 +33,9 @@ class CircularTubularStructure(GeometricalStructure):
     """
 
     def get_params_from_settings(self, single_structure_settings):
-        params = (np.asarray(single_structure_settings[Tags.STRUCTURE_START_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_END_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_RADIUS_MM]),
+        params = (single_structure_settings[Tags.STRUCTURE_START_MM],
+                  single_structure_settings[Tags.STRUCTURE_END_MM],
+                  single_structure_settings[Tags.STRUCTURE_RADIUS_MM],
                   single_structure_settings[Tags.CONSIDER_PARTIAL_VOLUME])
         return params
 
@@ -46,14 +48,17 @@ class CircularTubularStructure(GeometricalStructure):
 
     def get_enclosed_indices(self):
         start_mm, end_mm, radius_mm, partial_volume = self.params
+        start_mm = torch.tensor(start_mm, dtype=torch.float).to(self.torch_device)
+        end_mm = torch.tensor(end_mm, dtype=torch.float).to(self.torch_device)
+        radius_mm = torch.tensor(radius_mm, dtype=torch.float).to(self.torch_device)
         start_voxels = start_mm / self.voxel_spacing
         end_voxels = end_mm / self.voxel_spacing
         radius_voxels = radius_mm / self.voxel_spacing
 
-        x, y, z = np.meshgrid(np.arange(self.volume_dimensions_voxels[0]),
-                              np.arange(self.volume_dimensions_voxels[1]),
-                              np.arange(self.volume_dimensions_voxels[2]),
-                              indexing='ij')
+        x, y, z = torch.meshgrid(torch.arange(self.volume_dimensions_voxels[0]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[1]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[2]).to(self.torch_device),
+                                 indexing='ij')
 
         x = x + 0.5
         y = y + 0.5
@@ -64,30 +69,32 @@ class CircularTubularStructure(GeometricalStructure):
         else:
             radius_margin = 0.7071
 
-        target_vector = np.subtract(np.stack([x, y, z], axis=-1), start_voxels)
+        target_vector = torch.subtract(torch.stack([x, y, z], axis=-1), start_voxels)
         if self.do_deformation:
             # the deformation functional needs mm as inputs and returns the result in reverse indexing order...
-            deformation_values_mm = self.deformation_functional_mm(np.arange(self.volume_dimensions_voxels[0], step=1) *
+            deformation_values_mm = self.deformation_functional_mm(torch.arange(self.volume_dimensions_voxels[0]) *
                                                                    self.voxel_spacing,
-                                                                   np.arange(self.volume_dimensions_voxels[1], step=1) *
+                                                                   torch.arange(self.volume_dimensions_voxels[1]) *
                                                                    self.voxel_spacing).T
             deformation_values_mm = deformation_values_mm.reshape(self.volume_dimensions_voxels[0],
                                                                   self.volume_dimensions_voxels[1], 1, 1)
-            deformation_values_mm = np.tile(deformation_values_mm, (1, 1, self.volume_dimensions_voxels[2], 3))
-            target_vector = target_vector + (deformation_values_mm / self.voxel_spacing)
-        cylinder_vector = np.subtract(end_voxels, start_voxels)
+            deformation_values_mm = torch.tile(torch.from_numpy(deformation_values_mm).to(
+                self.torch_device), (1, 1, self.volume_dimensions_voxels[2], 3))
+            target_vector = (target_vector + (deformation_values_mm / self.voxel_spacing)).float()
+        cylinder_vector = torch.subtract(end_voxels, start_voxels)
 
-        target_radius = np.linalg.norm(target_vector, axis=-1) * np.sin(
-            np.arccos((np.dot(target_vector, cylinder_vector)) /
-                      (np.linalg.norm(target_vector, axis=-1) * np.linalg.norm(cylinder_vector))))
+        target_radius = torch.linalg.norm(target_vector, axis=-1) * torch.sin(
+            torch.arccos((torch.matmul(target_vector, cylinder_vector)) /
+                         (torch.linalg.norm(target_vector, axis=-1) * torch.linalg.norm(cylinder_vector))))
 
-        volume_fractions = np.zeros(self.volume_dimensions_voxels)
+        volume_fractions = torch.zeros(tuple(self.volume_dimensions_voxels), dtype=torch.float).to(self.torch_device)
 
         filled_mask = target_radius <= radius_voxels - 1 + radius_margin
         border_mask = (target_radius > radius_voxels - 1 + radius_margin) & \
                       (target_radius < radius_voxels + 2 * radius_margin)
 
         volume_fractions[filled_mask] = 1
+
         volume_fractions[border_mask] = 1 - (target_radius - (radius_voxels - radius_margin))[border_mask]
         volume_fractions[volume_fractions < 0] = 0
 
@@ -96,7 +103,7 @@ class CircularTubularStructure(GeometricalStructure):
         else:
             mask = filled_mask
 
-        return mask, volume_fractions[mask]
+        return mask.cpu().numpy(), volume_fractions[mask].cpu().numpy()
 
 
 def define_circular_tubular_structure_settings(tube_start_mm: list,
@@ -118,4 +125,4 @@ def define_circular_tubular_structure_settings(tube_start_mm: list,
         Tags.CONSIDER_PARTIAL_VOLUME: consider_partial_volume,
         Tags.ADHERE_TO_DEFORMATION: adhere_to_deformation,
         Tags.STRUCTURE_TYPE: Tags.CIRCULAR_TUBULAR_STRUCTURE
-    }
\ No newline at end of file
+    }
