@@ -2,7 +2,7 @@
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
 
-import numpy as np
+import torch
 
 from simpa.utils import Tags
 from simpa.utils.libraries.molecule_library import MolecularComposition
@@ -35,11 +35,11 @@ class EllipticalTubularStructure(GeometricalStructure):
     """
 
     def get_params_from_settings(self, single_structure_settings):
-        params = (np.asarray(single_structure_settings[Tags.STRUCTURE_START_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_END_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_RADIUS_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_ECCENTRICITY]),
-                  np.asarray(single_structure_settings[Tags.CONSIDER_PARTIAL_VOLUME]))
+        params = (single_structure_settings[Tags.STRUCTURE_START_MM],
+                  single_structure_settings[Tags.STRUCTURE_END_MM],
+                  single_structure_settings[Tags.STRUCTURE_RADIUS_MM],
+                  single_structure_settings[Tags.STRUCTURE_ECCENTRICITY],
+                  single_structure_settings[Tags.CONSIDER_PARTIAL_VOLUME])
         return params
 
     def to_settings(self):
@@ -53,15 +53,20 @@ class EllipticalTubularStructure(GeometricalStructure):
 
     def get_enclosed_indices(self):
         start_mm, end_mm, radius_mm, eccentricity, partial_volume = self.params
+        start_mm = torch.tensor(start_mm, dtype=torch.float).to(self.torch_device)
+        end_mm = torch.tensor(end_mm, dtype=torch.float).to(self.torch_device)
+        radius_mm = torch.tensor(radius_mm, dtype=torch.float).to(self.torch_device)
+        eccentricity = torch.tensor(eccentricity, dtype=torch.float).to(self.torch_device)
+        partial_volume = torch.tensor(partial_volume, dtype=torch.float).to(self.torch_device)
 
         start_voxels = start_mm / self.voxel_spacing
         end_voxels = end_mm / self.voxel_spacing
         radius_voxels = radius_mm / self.voxel_spacing
 
-        x, y, z = np.meshgrid(np.arange(self.volume_dimensions_voxels[0]),
-                              np.arange(self.volume_dimensions_voxels[1]),
-                              np.arange(self.volume_dimensions_voxels[2]),
-                              indexing='ij')
+        x, y, z = torch.meshgrid(torch.arange(self.volume_dimensions_voxels[0]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[1]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[2]).to(self.torch_device),
+                                 indexing='ij')
 
         x = x + 0.5
         y = y + 0.5
@@ -72,40 +77,41 @@ class EllipticalTubularStructure(GeometricalStructure):
         else:
             radius_margin = 0.7071
 
-        target_vector = np.subtract(np.stack([x, y, z], axis=-1), start_voxels)
+        target_vector = torch.subtract(torch.stack([x, y, z], axis=-1), start_voxels)
         if self.do_deformation:
             # the deformation functional needs mm as inputs and returns the result in reverse indexing order...
-            deformation_values_mm = self.deformation_functional_mm(np.arange(self.volume_dimensions_voxels[0], step=1) *
+            deformation_values_mm = self.deformation_functional_mm(torch.arange(self.volume_dimensions_voxels[0]) *
                                                                    self.voxel_spacing,
-                                                                   np.arange(self.volume_dimensions_voxels[1], step=1) *
+                                                                   torch.arange(self.volume_dimensions_voxels[1]) *
                                                                    self.voxel_spacing).T
             deformation_values_mm = deformation_values_mm.reshape(self.volume_dimensions_voxels[0],
                                                                   self.volume_dimensions_voxels[1], 1, 1)
-            deformation_values_mm = np.tile(deformation_values_mm, (1, 1, self.volume_dimensions_voxels[2], 3))
-            target_vector = target_vector + (deformation_values_mm / self.voxel_spacing)
-        cylinder_vector = np.subtract(end_voxels, start_voxels)
+            deformation_values_mm = torch.tile(torch.from_numpy(deformation_values_mm).to(
+                self.torch_device), (1, 1, self.volume_dimensions_voxels[2], 3))
+            target_vector = (target_vector + (deformation_values_mm / self.voxel_spacing)).float()
+        cylinder_vector = torch.subtract(end_voxels, start_voxels)
 
         main_axis_length = radius_voxels/(1-eccentricity**2)**0.25
-        main_axis_vector = np.array([cylinder_vector[1], -cylinder_vector[0], 0])
-        main_axis_vector = main_axis_vector/np.linalg.norm(main_axis_vector) * main_axis_length
+        main_axis_vector = torch.tensor([cylinder_vector[1], -cylinder_vector[0], 0]).to(self.torch_device)
+        main_axis_vector = main_axis_vector/torch.linalg.norm(main_axis_vector) * main_axis_length
 
-        minor_axis_length = main_axis_length*np.sqrt(1-eccentricity**2)
-        minor_axis_vector = np.cross(cylinder_vector, main_axis_vector)
-        minor_axis_vector = minor_axis_vector / np.linalg.norm(minor_axis_vector) * minor_axis_length
+        minor_axis_length = main_axis_length*torch.sqrt(1-eccentricity**2)
+        minor_axis_vector = torch.cross(cylinder_vector, main_axis_vector)
+        minor_axis_vector = minor_axis_vector / torch.linalg.norm(minor_axis_vector) * minor_axis_length
 
-        dot_product = np.dot(target_vector, cylinder_vector)/np.linalg.norm(cylinder_vector)
+        dot_product = torch.matmul(target_vector, cylinder_vector)/torch.linalg.norm(cylinder_vector)
 
-        target_vector_projection = np.multiply(dot_product[:, :, :, np.newaxis], cylinder_vector)
+        target_vector_projection = torch.multiply(dot_product[:, :, :, None], cylinder_vector)
         target_vector_from_projection = target_vector - target_vector_projection
 
-        main_projection = np.dot(target_vector_from_projection, main_axis_vector) / main_axis_length
+        main_projection = torch.matmul(target_vector_from_projection, main_axis_vector) / main_axis_length
 
-        minor_projection = np.dot(target_vector_from_projection, minor_axis_vector) / minor_axis_length
+        minor_projection = torch.matmul(target_vector_from_projection, minor_axis_vector) / minor_axis_length
 
-        radius_crit = np.sqrt(((main_projection/main_axis_length)**2 + (minor_projection/minor_axis_length)**2) *
-                              radius_voxels**2)
+        radius_crit = torch.sqrt(((main_projection/main_axis_length)**2 + (minor_projection/minor_axis_length)**2) *
+                                 radius_voxels**2)
 
-        volume_fractions = np.zeros(self.volume_dimensions_voxels)
+        volume_fractions = torch.zeros(tuple(self.volume_dimensions_voxels), dtype=torch.float).to(self.torch_device)
         filled_mask = radius_crit <= radius_voxels - 1 + radius_margin
         border_mask = (radius_crit > radius_voxels - 1 + radius_margin) & \
                       (radius_crit < radius_voxels + 2 * radius_margin)
@@ -121,7 +127,7 @@ class EllipticalTubularStructure(GeometricalStructure):
         else:
             mask = filled_mask
 
-        return mask, volume_fractions[mask]
+        return mask.cpu().numpy(), volume_fractions[mask].cpu().numpy()
 
 
 def define_elliptical_tubular_structure_settings(tube_start_mm: list,
@@ -145,4 +151,4 @@ def define_elliptical_tubular_structure_settings(tube_start_mm: list,
         Tags.CONSIDER_PARTIAL_VOLUME: consider_partial_volume,
         Tags.ADHERE_TO_DEFORMATION: adhere_to_deformation,
         Tags.STRUCTURE_TYPE: Tags.ELLIPTICAL_TUBULAR_STRUCTURE
-    }
\ No newline at end of file
+    }
