@@ -2,7 +2,7 @@
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
 
-import numpy as np
+import torch
 
 from simpa.utils import Tags
 from simpa.utils.libraries.molecule_library import MolecularComposition
@@ -29,10 +29,10 @@ class ParallelepipedStructure(GeometricalStructure):
     """
 
     def get_params_from_settings(self, single_structure_settings):
-        params = (np.asarray(single_structure_settings[Tags.STRUCTURE_START_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_FIRST_EDGE_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_SECOND_EDGE_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_THIRD_EDGE_MM]))
+        params = (single_structure_settings[Tags.STRUCTURE_START_MM],
+                  single_structure_settings[Tags.STRUCTURE_FIRST_EDGE_MM],
+                  single_structure_settings[Tags.STRUCTURE_SECOND_EDGE_MM],
+                  single_structure_settings[Tags.STRUCTURE_THIRD_EDGE_MM])
         return params
 
     def to_settings(self):
@@ -45,36 +45,39 @@ class ParallelepipedStructure(GeometricalStructure):
 
     def get_enclosed_indices(self):
         start_mm, x_edge_mm, y_edge_mm, z_edge_mm = self.params
+        start_mm = torch.tensor(start_mm, dtype=torch.float).to(self.torch_device)
+        x_edge_mm = torch.tensor(x_edge_mm, dtype=torch.float).to(self.torch_device)
+        y_edge_mm = torch.tensor(y_edge_mm, dtype=torch.float).to(self.torch_device)
+        z_edge_mm = torch.tensor(z_edge_mm, dtype=torch.float).to(self.torch_device)
+
         start_voxels = start_mm / self.voxel_spacing
-        x_edge_voxels = np.array(x_edge_mm / self.voxel_spacing)
-        y_edge_voxels = np.array(y_edge_mm / self.voxel_spacing)
-        z_edge_voxels = np.array(z_edge_mm / self.voxel_spacing)
+        x_edge_voxels = x_edge_mm / self.voxel_spacing
+        y_edge_voxels = y_edge_mm / self.voxel_spacing
+        z_edge_voxels = z_edge_mm / self.voxel_spacing
 
-        x, y, z = np.meshgrid(np.arange(self.volume_dimensions_voxels[0]),
-                              np.arange(self.volume_dimensions_voxels[1]),
-                              np.arange(self.volume_dimensions_voxels[2]),
-                              indexing='ij')
+        x, y, z = torch.meshgrid(torch.arange(self.volume_dimensions_voxels[0]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[1]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[2]).to(self.torch_device),
+                                 indexing='ij')
 
-        target_vector = np.subtract(np.stack([x, y, z], axis=-1), start_voxels)
+        target_vector = torch.subtract(torch.stack([x, y, z], axis=-1), start_voxels)
 
-        matrix = np.array([x_edge_voxels, y_edge_voxels, z_edge_voxels])
+        matrix = torch.stack((x_edge_voxels, y_edge_voxels, z_edge_voxels))
 
-        inverse_matrix = np.linalg.inv(matrix)
+        result = torch.linalg.solve(matrix.T.expand((target_vector.shape[:-1]+matrix.shape)), target_vector)
 
-        result = np.matmul(target_vector, inverse_matrix)
-
-        norm_vector = np.array([1/np.linalg.norm(x_edge_voxels),
-                                1/np.linalg.norm(y_edge_voxels),
-                                1/np.linalg.norm(z_edge_voxels)])
+        norm_vector = torch.tensor([1/torch.linalg.norm(x_edge_voxels),
+                                    1/torch.linalg.norm(y_edge_voxels),
+                                    1/torch.linalg.norm(z_edge_voxels)]).to(self.torch_device)
 
         filled_mask_bool = (0 <= result) & (result <= 1 - norm_vector)
 
-        volume_fractions = np.zeros(self.volume_dimensions_voxels)
-        filled_mask = np.all(filled_mask_bool, axis=-1)
+        volume_fractions = torch.zeros(tuple(self.volume_dimensions_voxels), dtype=torch.float).to(self.torch_device)
+        filled_mask = torch.all(filled_mask_bool, axis=-1)
 
         volume_fractions[filled_mask] = 1
 
-        return filled_mask, volume_fractions[filled_mask]
+        return filled_mask.cpu().numpy(), volume_fractions[filled_mask].cpu().numpy()
 
 
 def define_parallelepiped_structure_settings(start_mm: list, edge_a_mm: list, edge_b_mm: list, edge_c_mm: list,
@@ -95,4 +98,4 @@ def define_parallelepiped_structure_settings(start_mm: list, edge_a_mm: list, ed
         Tags.CONSIDER_PARTIAL_VOLUME: consider_partial_volume,
         Tags.ADHERE_TO_DEFORMATION: adhere_to_deformation,
         Tags.STRUCTURE_TYPE: Tags.PARALLELEPIPED_STRUCTURE
-    }
\ No newline at end of file
+    }
