@@ -2,7 +2,8 @@
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
 
-import numpy as np
+from typing import Union
+import torch
 
 from simpa.utils import Tags
 from simpa.utils.libraries.molecule_library import MolecularComposition
@@ -33,10 +34,10 @@ class RectangularCuboidStructure(GeometricalStructure):
     """
 
     def get_params_from_settings(self, single_structure_settings):
-        params = (np.asarray(single_structure_settings[Tags.STRUCTURE_START_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_X_EXTENT_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_Y_EXTENT_MM]),
-                  np.asarray(single_structure_settings[Tags.STRUCTURE_Z_EXTENT_MM]),
+        params = (single_structure_settings[Tags.STRUCTURE_START_MM],
+                  single_structure_settings[Tags.STRUCTURE_X_EXTENT_MM],
+                  single_structure_settings[Tags.STRUCTURE_Y_EXTENT_MM],
+                  single_structure_settings[Tags.STRUCTURE_Z_EXTENT_MM],
                   single_structure_settings[Tags.CONSIDER_PARTIAL_VOLUME])
         return params
 
@@ -51,41 +52,46 @@ class RectangularCuboidStructure(GeometricalStructure):
 
     def get_enclosed_indices(self):
         start_mm, x_edge_mm, y_edge_mm, z_edge_mm, partial_volume = self.params
+        start_mm = torch.tensor(start_mm, dtype=torch.float).to(self.torch_device)
+        x_edge_mm = torch.tensor(x_edge_mm, dtype=torch.float).to(self.torch_device)
+        y_edge_mm = torch.tensor(y_edge_mm, dtype=torch.float).to(self.torch_device)
+        z_edge_mm = torch.tensor(z_edge_mm, dtype=torch.float).to(self.torch_device)
+
         start_voxels = start_mm / self.voxel_spacing
-        x_edge_voxels = np.array([x_edge_mm / self.voxel_spacing, 0, 0])
-        y_edge_voxels = np.array([0, y_edge_mm / self.voxel_spacing, 0])
-        z_edge_voxels = np.array([0, 0, z_edge_mm / self.voxel_spacing])
+        x_edge_voxels = torch.tensor([x_edge_mm / self.voxel_spacing, 0, 0]).to(self.torch_device)
+        y_edge_voxels = torch.tensor([0, y_edge_mm / self.voxel_spacing, 0]).to(self.torch_device)
+        z_edge_voxels = torch.tensor([0, 0, z_edge_mm / self.voxel_spacing]).to(self.torch_device)
 
-        x, y, z = np.meshgrid(np.arange(self.volume_dimensions_voxels[0]),
-                              np.arange(self.volume_dimensions_voxels[1]),
-                              np.arange(self.volume_dimensions_voxels[2]),
-                              indexing='ij')
+        x, y, z = torch.meshgrid(torch.arange(self.volume_dimensions_voxels[0]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[1]).to(self.torch_device),
+                                 torch.arange(self.volume_dimensions_voxels[2]).to(self.torch_device),
+                                 indexing='ij')
 
-        target_vector = np.subtract(np.stack([x, y, z], axis=-1), start_voxels)
+        target_vector = torch.subtract(torch.stack([x, y, z], axis=-1), start_voxels)
 
-        matrix = np.array([x_edge_voxels, y_edge_voxels, z_edge_voxels])
+        matrix = torch.stack([x_edge_voxels, y_edge_voxels, z_edge_voxels])
 
-        inverse_matrix = np.linalg.inv(matrix)
+        inverse_matrix = torch.linalg.inv(matrix)
 
-        result = np.matmul(target_vector, inverse_matrix)
+        result = torch.matmul(target_vector, inverse_matrix)
 
-        norm_vector = np.array([1/np.linalg.norm(x_edge_voxels),
-                                1/np.linalg.norm(y_edge_voxels),
-                                1/np.linalg.norm(z_edge_voxels)])
+        norm_vector = torch.tensor([1/torch.linalg.norm(x_edge_voxels),
+                                    1/torch.linalg.norm(y_edge_voxels),
+                                    1/torch.linalg.norm(z_edge_voxels)]).to(self.torch_device)
 
         filled_mask_bool = (0 <= result) & (result <= 1 - norm_vector)
         border_bool = (0 - norm_vector < result) & (result <= 1)
 
-        volume_fractions = np.zeros(self.volume_dimensions_voxels)
-        filled_mask = np.all(filled_mask_bool, axis=-1)
+        volume_fractions = torch.zeros(tuple(self.volume_dimensions_voxels), dtype=torch.float).to(self.torch_device)
+        filled_mask = torch.all(filled_mask_bool, axis=-1)
 
-        border_mask = np.all(border_bool, axis=-1)
+        border_mask = torch.all(border_bool, axis=-1)
 
-        border_mask = np.logical_xor(border_mask, filled_mask)
+        border_mask = torch.logical_xor(border_mask, filled_mask)
 
         edge_values = result[border_mask]
 
-        fraction_values = np.matmul(edge_values, matrix)
+        fraction_values = torch.matmul(edge_values, matrix)
 
         larger_fraction_values = (x_edge_voxels + y_edge_voxels + z_edge_voxels) - fraction_values
 
@@ -96,7 +102,7 @@ class RectangularCuboidStructure(GeometricalStructure):
         fraction_values[fraction_values <= 0] = 1 + fraction_values[fraction_values <= 0]
         fraction_values[larger_fraction_values < 1] = larger_fraction_values[larger_fraction_values < 1]
 
-        fraction_values = np.abs(np.prod(fraction_values, axis=-1))
+        fraction_values = torch.abs(torch.prod(fraction_values, axis=-1))
 
         volume_fractions[filled_mask] = 1
         volume_fractions[border_mask] = fraction_values
@@ -106,10 +112,10 @@ class RectangularCuboidStructure(GeometricalStructure):
         else:
             mask = filled_mask
 
-        return mask, volume_fractions[mask]
+        return mask.cpu().numpy(), volume_fractions[mask].cpu().numpy()
 
 
-def define_rectangular_cuboid_structure_settings(start_mm: list, extent_mm: (int, list),
+def define_rectangular_cuboid_structure_settings(start_mm: list, extent_mm: Union[int, list],
                                                  molecular_composition: MolecularComposition, priority: int = 10,
                                                  consider_partial_volume: bool = False,
                                                  adhere_to_deformation: bool = False):
@@ -129,4 +135,4 @@ def define_rectangular_cuboid_structure_settings(start_mm: list, extent_mm: (int
         Tags.CONSIDER_PARTIAL_VOLUME: consider_partial_volume,
         Tags.ADHERE_TO_DEFORMATION: adhere_to_deformation,
         Tags.STRUCTURE_TYPE: Tags.RECTANGULAR_CUBOID_STRUCTURE
-    }
\ No newline at end of file
+    }
